#!/usr/bin/env bash
set -euo pipefail
IFS=$'\n'

# For when script is called directly
if [ "${ENABLE_HELM3-false}" = "true" ]; then
  export PATH=/usr/local/bin/helm3/:/usr/local/bin/helmsman3/:$PATH
fi

# # Ignore any ENV variables for AWS Auth and use default profile with the assume CICD Role
# unset AWS_ACCESS_KEY_ID
# unset AWS_SECRET_ACCESS_KEY
# unset AWS_SESSION_TOKEN

# shellcheck source=src/bin/mosaic-common.bash
source "$(dirname "${BASH_SOURCE[0]}")/mosaic-common.bash"
set_usage "Mosaic helmsman Wrapper" "
Run helmsman against any state files in the given directory. Any \"*.yaml\" or
\"*.toml\" file in the given directory is treated as a helmsman state file.

Searches for subdirectories that contain a \"Chart.yaml\" file and runs helm
linting against any charts found.
"

function helm_directories {
  local under="${1:-.}"
  find -L "${under}" \( -name '.*' \! -name '.' \) -prune \
    -o -name 'Chart.yaml' -type f -exec dirname \{\} \; | \
    sort -u
}

function helm_check {
  local directory="$1"

  (
    helmsman version
    helm version --short
    if_verbose cd "${directory}"
    if_verbose helm lint
  )
}

function helmsman_check {
  local directory="$1"

  (
    helmsman version
    helm version --short
    if_verbose cd "${directory}"
    shopt -s nullglob
    for state_file in *.yaml *.toml; do
      if_verbose helmsman --no-ns --verbose -e env/"${MOSAIC_HELMSMAN_ENV}".yaml -f "${state_file}"
    done
  )
}

function helmsman_show {
  local directory="$1"

  (
    echo "==> DEPRECATED APIs <=="
    echo "See https://confluence.carbonblack.local/display/MOSAIC/Upcoming+Kubernetes+API+Removals for more details"
    if [ "${ENABLE_HELM3-false}" = "true" ]
    then
      if_verbose pluto --kubeconfig ~/.kube/config detect-helm -n "$APPLICATION" -t k8s=v1.16.0 --ignore-deprecations --ignore-removals || true
    else
      if_verbose pluto --kubeconfig ~/.kube/config detect-helm --helm-version 2 --helm-store secrets -n "$TILLER_NAMESPACE" -t k8s=v1.16.0 --ignore-deprecations --ignore-removals || true
    fi
    if_verbose cd "${directory}"
    shopt -s nullglob
    for state_file in *.yaml *.toml; do
      if_verbose helmsman --no-ns --verbose --dry-run -e env/"${MOSAIC_HELMSMAN_ENV}".yaml -f "${state_file}"
    done
  )
}

function helmsman_deploy {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    shopt -s nullglob
    for state_file in *.yaml *.toml; do
      if_verbose helmsman --no-ns --verbose --apply -e env/"${MOSAIC_HELMSMAN_ENV}".yaml -f "${state_file}"
    done
  )
}

function helmsman_destroy {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    shopt -s nullglob
    for state_file in *.yaml *.toml; do
      if_verbose helmsman --destroy -e env/"${MOSAIC_HELMSMAN_ENV}".yaml -f "${state_file}"
    done
  )
}

mosaic-configure.bash
unset_AWS_env_creds
parse_args "$@"

case "${action}" in
  check)
    for d in $(helm_directories "${directory}"); do
      helm_check "${d}"
    done
    helmsman_check "${directory}"
    ;;
  show)
    ## Remove when helm2to3 migrations are done
    trap 'catch $?' EXIT
    # shellcheck source=src/bin/helm2to3.bash
    source "$(dirname "${BASH_SOURCE[0]}")/helm2to3.bash"
    if [ "${HELM2TO3-false}" = "true" ]
    then
      helm2to3_show
    else
      helmsman_show "${directory}"
    fi
    ##
    # helmsman_show "${directory}" ## Uncomment this line when helm2to3 migrations are done
    ;;
  deploy)
    ## Remove when helm2to3 migrations are done
    trap 'catch $?' EXIT
    # shellcheck source=src/bin/helm2to3.bash
    source "$(dirname "${BASH_SOURCE[0]}")/helm2to3.bash"
    if [ "${HELM2TO3-false}" = "true" ]
    then
      helm2to3_deploy
    else
      helmsman_deploy "${directory}"
    fi
    ##
    # helmsman_deploy "${directory}" ## Uncomment this line when helm2to3 migrations are done
    ;;
  destroy)
    helmsman_destroy "${directory}"
    ;;
  *)
    usage
    ;;
esac
