#!/usr/bin/env python3

import dataclasses
import json
import os
from pathlib import Path
import time
import sys

import botocore
import boto3
import requests


@dataclasses.dataclass
class Text:
    type_: str
    emoji: bool = False
    text: str = ""

    def reprJSON(self):
        r = {"type": self.type_}

        if self.emoji:
            r["emoji"] = self.emoji

        if self.text:
            r["text"] = self.text

        return r


@dataclasses.dataclass
class Blocks:
    type_: str
    text: str = ""

    def reprJSON(self):
        r = {"type": self.type_, "text": self.text}
        if r["text"] == "":
            del r["text"]

        return r


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):
        if hasattr(obj, "reprJSON"):
            return obj.reprJSON()
        else:
            return json.JSONEncoder.default(self, obj)


class NotifySlack:
    def __init__(self):
        self.session = requests.Session()
        self.dynamodb = boto3.resource("dynamodb")
        self.dynamodb_table = self.dynamodb.Table(
            "mosaic-mosaic-prod-slack-notify-lock"
        )
        self.lock_path = "/tmp/mosaic_slack_notified"
        self.slack_webhook_url = self.getWebhookSecret()

    def getWebhookSecret(self):
        webhook_secret = open("/etc/mosaic/slack_webhook").read().strip()
        return f"https://hooks.slack.com/services/{webhook_secret}"

    def getNeededEnvVars(self, name):
        value = os.environ.get(name)

        if value is None:
            print(
                f"!ERROR! Environment Var {name} is unset - not running deploy webhook"
            )
            sys.exit(0)

        return value

    def get_lock_in_dynamo(self, lockId):
        # One post every 10 minutes for a given pipeline
        ttl = int(time.time()) + 10 * 60

        response = self.dynamodb_table.get_item(
            Key={"LockID": lockId}, ConsistentRead=True
        )

        item = response.get("Item")

        # Lock not found - get it
        if item is None:
            return self.set_lock_in_dynamodb(lockId, ttl)

        # A lock exists check if it is expired
        existing_ttl = item.get("TTL", 0)

        if time.time() > existing_ttl:
            return self.set_lock_in_dynamodb(lockId, ttl, override=True)

        return False

    def set_lock_in_dynamodb(self, lockId, ttl, override=False):
        lockItem = {"LockID": lockId, "TTL": ttl}

        if override:
            self.dynamodb_table.put_item(Item=lockItem)
            return True

        try:
            self.dynamodb_table.put_item(
                Item=lockItem, ConditionExpression="attribute_not_exists(lockId)"
            )
            return True
        except botocore.exceptions.ClientError as e:
            # Something else got the lock before this process did.
            if e.response["Error"]["Code"] != "ConditionalCheckFailedException":
                raise
            return False

    def main(self):
        if "DISABLE_MOSAIC_SLACK_WEBHOOK" in os.environ:
            print("Slack webhook disabled")
            sys.exit(0)

        if Path(self.lock_path).exists():
            sys.exit(0)

        Path(self.lock_path).touch()

        env_category = self.getNeededEnvVars("ENV_CATEGORY")

        if env_category != "prod":
            print("ENV_CATEGORY is not prod - not running slack webhook")
            sys.exit(0)

        application = self.getNeededEnvVars("APPLICATION")
        region = self.getNeededEnvVars("REGION")
        product = self.getNeededEnvVars("PRODUCT")

        pipeline_id = self.getNeededEnvVars("CI_PIPELINE_ID")
        pipeline_url = self.getNeededEnvVars("CI_PIPELINE_URL")
        project_url = self.getNeededEnvVars("CI_PROJECT_URL")
        project_path = self.getNeededEnvVars("CI_PROJECT_PATH")
        commit_short_sha = self.getNeededEnvVars("CI_COMMIT_SHORT_SHA")
        commit_title = self.getNeededEnvVars("CI_COMMIT_TITLE")

        message = {
            "blocks": [
                Blocks(
                    type_="section",
                    text=Text(
                        type_="mrkdwn",
                        emoji=False,
                        text=f"App *{product}-{application}* pipeline deploying to production.",
                    ),
                ),
                Blocks(
                    type_="section",
                    text=Text(
                        type_="mrkdwn",
                        text=f"*Subject:* {commit_title}\n<{pipeline_url}|GitLab CI Pipeline> *|* <{project_url}/-/commit/{commit_short_sha}|{commit_short_sha}>",
                    ),
                ),
                Blocks(type_="divider"),
            ]
        }

        lockId = f"{project_path}/{pipeline_id}"

        lock_status = self.get_lock_in_dynamo(lockId)

        # Did not achieve lock - exit.
        if not lock_status:
            # print("NOT POSTING TO SLACK")
            sys.exit(0)

        # print("POSTING TO SLACK")

        self.session.post(
            self.slack_webhook_url,
            data=json.dumps(message, cls=ComplexEncoder),
            headers={"Content-Type": "application/json"},
            timeout=15,
        )


if __name__ == "__main__":
    try:
        ns = NotifySlack()
        ns.main()
    except SystemExit:
        pass
    except Exception as e:
        print(e)
        print("Error calling Slack... ignoring. Reach out to #mosaic-support")
        if os.environ.get("MOSAIC_DEBUG_SLACK"):
            raise
